/*
mondo-tarme                                                Nov 9th, 2001

11/09
- if differences found during verification of CD then don't say the CD is bad;
  just say, differences were found :-)

11/06
- res+=retval; should have been retval+=res (Tony Kim)

10/31
- moved call to verify_cd_image() to the line after make_iso_fs; if verify
  fails, Mondo will prompt user to retry (burning the CD)

10/15
- disabled the 'burning first/last/whatever CD' comment; now it's just, plain
  'burnin CD #n'; much easier to code & less potential for confusion

10/13
- if user uses --write-tapes and --verify 1|2|3 then tell user Mondo doesn't
  yet know how to verify tape backups

10/11
- replace USE_BZIP2 with USE_LZO boolean; bzip2 is default; LZO is faster
  option (NB: _option_) activated by '--use-lzo' command-line parameter
- replaced '.bz2' with ZIPPER_SUFFIX
- replaced bzip2 with ZIPPER_EXECUTABLE
- slices are always bzip2'd; never lzo'd

10/06
- if fails but user doesn't abort then _continue_; don't loop around & around

10/03
- if backup (CD #n) fails, offer to retry; then, offer to abort; if user
  chooses not to retry but not to abort either then continue

09/30
- writing make_tape_archives(), to backup stuff to tape

09/26
- measure progress by the filelist# we're on, rather than the number of
  files that we have archived so far

09/17
- working on CD-verifying routines
- if verify is enabled then CD will not be ejected after being burnt

09/09
- added a call to verify the CD images after creating them

09/06
- ported mondo-tarme to C; this C source was created on 09/03/2001


-----------------------------------------------------------------------------
*/


#include "my-stuff.h"



char MONDO_LOGFILE[MAX_STR_LEN]="/var/log/mondo-archive.log", // default
  CALL_BEFORE_ISO[MAX_STR_LEN],
  CALL_MAKE_ISO[MAX_STR_LEN],
  CALL_BURN_ISO[MAX_STR_LEN],
  CALL_AFTER_ISO[MAX_STR_LEN],
  TMP[MAX_STR_LEN],
  STARTDIR[MAX_STR_LEN],
  MONDO_PATH[MAX_STR_LEN],
  TAPE_DEVICE[MAX_STR_LEN],
  ZIPPER_EXECUTABLE[MAX_STR_LEN],
  ZIPPER_SUFFIX[MAX_STR_LEN];

int CURR_ISO_NO,COMPRESSION_LEVEL,VERIFY_LEVEL;
long OPTIMAL_SET_SIZE,TAPE_SIZE;
bool DEBUGGING, RUNNING_LIVE, USE_LZO;

extern long g_start_time, g_minimum_progress, g_maximum_progress, 
  g_current_progress, g_currentY;     // declared in mondo-tools.c
extern char err_log_lines[NOOF_ERR_LINES][MAX_STR_LEN];
extern void center_string(char*,int);

/* ---------------------- externs -------------------- */

extern void finish(int);
extern void setup_newt_stuff(void);
extern int ask_me_yes_or_no(char*);
extern void log_to_screen(char*);
extern void update_progress_form(char*);
extern void open_progress_form(char*,char*,char*,char*,long);
extern void close_progress_form(void);
extern void popup_and_OK(char*);
extern long get_time(void);
extern int run_program_and_log_output(char*);
extern int strcmp_inc_numbers(char*,char*);
extern long count_lines_in_file(char*);
extern long length_of_file(char*);
extern char *slice_fname(long,long,bool,char*);
extern bool does_file_exist(char*);
extern char *last_line_of_file(char*);
extern void log_file_end_to_screen(char*,char*);
extern void open_evalcall_form(char*);
extern void update_evalcall_form(int);
extern void close_evalcall_form(void);


/* --------------------- my routines ----------------- */

int add_files_to_cd(char*,char*,long,char list_of_files[ARBITRARY_MAXIMUM][MAX_STR_LEN],int);
void resolve_naff_tokens(char*,char*,char*,char*);
int eval_call(char*,char*,int,char*,char*);
void fatal_error(char*);
int make_iso_fs(char*,char*);
int make_tarballs_and_images(char*,char*,long);
int make_slices_and_images(char*,char*,long,char*);
int make_those_mondo_archives_dude(char*,char*,long);
void pause_and_ask_for_cdr(int);
int send_slices_to_cd(char*,char*,long,long);
int slice_up_file(char*,long,char*,long);
long space_occupied_by_cd(char*);
int tar_this_set(char*,char*,int);
int write_final_iso_if_necessary(char*,char*,long);
int write_iso_and_go_on(char*,char*,long,bool);
void wipe_archives(char*);
char *trim_empty_quotes(char*);



/*------------------------------ dummies ------------------------------*/

int what_number_cd_is_this(void) {log_it("FOO1");exit(100);}
int load_mountlist(struct mountlist_itself*a,char*b) {log_it("FOO2");exit(100);}
int make_list_of_drives(struct mountlist_itself*a, char **b) {log_it("FOO3");exit(100);}
//drivelist[ARBITRARY_MAXIMUM][MAX_STR_LEN]) {exit(100;}
void sort_mountlist_by_device(struct mountlist_itself*a,bool b) {log_it("FOO4");exit(100);}
void sort_mountlist_entries(struct mountlist_itself*a,bool b) {log_it("FOO5");exit(100);}
void swap_mountlist_entries(struct mountlist_itself*a,int b,int c){log_it("FOO6");exit(100);}

/*---------------------------------------------------------------------*/



int add_files_to_cd(char*scratchdir,char*isodir,long cdsize,char list_of_files[ARBITRARY_MAXIMUM][MAX_STR_LEN],int noof_files)
{
  int retval=0,res=0,i;
  char tmp[MAX_STR_LEN];
  long would_occupy;

  would_occupy=space_occupied_by_cd(scratchdir);
  for(i=0; i<noof_files; i++)
    {
      if (!does_file_exist(list_of_files[i]))
	{
	  sprintf(tmp,"Warning - you're trying to add a non-existent file - '%s' to the CD",list_of_files[i]);
	  log_it(tmp);
	}
      would_occupy+=length_of_file(list_of_files[i])/1024; 
    }
  if (would_occupy > cdsize)
    {
      sprintf(tmp,"OK, time to make CD #%d (%ld > %ld)",CURR_ISO_NO,would_occupy,cdsize);
      log_it(tmp);
      res=write_iso_and_go_on(scratchdir,isodir,cdsize,FALSE); 
      log_it("Back from write_iso_and_go_on");
      // ...,FALSE because this is not the last CD we'll write
      retval+=res;
      if (res) { log_it("WARNING - write_iso_and_go_on returned an error"); }
    }
  for(i=0;i<noof_files;i++)
    {
      sprintf(tmp,"cp -f %s %s/archives/",list_of_files[i],scratchdir);
      res=system(tmp);
      retval+=res;
      if (res) { strcat(tmp," ...failed"); log_to_screen(tmp); }
      else
	{
	  sprintf(tmp,"moved %s to CD dir OK",list_of_files[i]);
	  log_it(tmp);
	}
      unlink(list_of_files[i]);
    }
  if (retval)
    { log_it("Warning - errors occurred while I was adding files to CD dir"); }
  return(retval);
}



void resolve_naff_tokens(char*output,char*ip,char*value,char*token)
{
  char input[MAX_STR_LEN],tmp[MAX_STR_LEN],*p;
  strcpy(output,ip); // just in case the token doesn't appear in string at all
  for(strcpy(input,ip); strstr(input,token); strcpy(input,output))
    {
      strcpy(output,input);
      p=strstr(output,token);
      *p='\0';
      strcat(output,value);
      p=strstr(input,token) + strlen(token);
      strcat(output,p);
    }
}



int grab_percentage_from_last_line_of_file(char*filename)
{
  char tmp[MAX_STR_LEN],lastline[MAX_STR_LEN], *p;
  int i;
  //  strcpy(lastline,last_line_of_file(filename));
  strcpy(lastline,err_log_lines[NOOF_ERR_LINES-1]);
  sprintf(tmp,"lastline = %s",lastline);
  p=strstr(lastline,"% done");
  if (!p) {p=strstr(lastline,"%% Done");}
  if (!p) {p=strstr(lastline,"%% ");}
  if (!p) {return(0);}
  *p='\0';
  //  sprintf(tmp,"lastline = '%s'");
  //  log_to_screen(tmp);
  for(p-=1; *p!=' ' && p!=lastline; p--);
  if (p!=lastline) { p++; }
  i=atoi(p);
  //  sprintf(tmp,"'%s' --> %d",p,i);
  //  log_to_screen(tmp);
  return(i);
}




int eval_call(char*basic_call, char*isofile, int cd_no, char*logstub, char*what_i_am_doing)
{
  int retval=0,res=0,blanklines=0,jj,i;
  FILE*fin,*fout,*ferr;
  char midway_call[MAX_STR_LEN],ultimate_call[MAX_STR_LEN],tmp[MAX_STR_LEN],command[MAX_STR_LEN],incoming[MAX_STR_LEN],old_stderr[MAX_STR_LEN],stderr_output[MAX_STR_LEN],stderr_fname[MAX_STR_LEN], lockfile[MAX_STR_LEN], cd_number_str[MAX_STR_LEN], *p;
  old_stderr[0]='\0';
  sprintf(cd_number_str,"%d",cd_no);
  resolve_naff_tokens(midway_call,basic_call,isofile,"_ISO_");
  resolve_naff_tokens(tmp,midway_call,cd_number_str,"_CD#_");
  sprintf(stderr_fname,"%s/stderr.txt",TMP);
  resolve_naff_tokens(ultimate_call,tmp,stderr_fname,"_ERR_");
  sprintf(lockfile,"%s/mojo-jojo-for-president",TMP);
  sprintf(command,"echo hi > %s ; %s 2> %s; res=$?; rm -f %s; exit $res",lockfile,ultimate_call,stderr_fname,lockfile);

  /*
  if (VERIFY_LEVEL > 0)
    {
      p=strstr(command,"-eject ");
      if (p)
	{ for(i=0;i<6;i++) {p[i]=' ';}	}
    }
  */

  open_evalcall_form(what_i_am_doing);
  sprintf(tmp,"About to call '%s'...",command);
  log_it(tmp);
  fin=popen(command,"r");
  if (!fin)
    {
      sprintf(tmp,"Failed utterly to call '%s'",command);
      log_to_screen(tmp);
      return(1);
    }
  while(!does_file_exist(lockfile)) { sleep(1); }
  for(; does_file_exist(lockfile); sleep(1))
    {
      log_file_end_to_screen(stderr_fname,"");
      jj=grab_percentage_from_last_line_of_file(stderr_fname);
      if (jj>0) { update_evalcall_form(jj); }
    }
  for(fgets(incoming,MAX_STR_LEN,fin); !feof(fin); fgets(incoming,MAX_STR_LEN,fin))
    {
      incoming[78]='\0';
      while (incoming[strlen(incoming)-1]<32)
	{ incoming[strlen(incoming)-1]='\0'; }
      if (strlen(incoming)==0) 
	{
	blanklines++;
	//	if (blanklines>5) {break;}
	}
      log_to_screen(incoming);
    }
  if (pclose(fin)) {res=1;}
  retval+=res;
  close_evalcall_form();
  unlink(stderr_fname);
  return(retval);
}



void fatal_error(char*error_string)
{
  char fatalstr[MAX_STR_LEN]="-------mondo-tarme----------FATAL ERROR---------mondo-tarme----------";
  log_it(fatalstr);
  log_it(error_string);
  log_it("Mondo-tarme has stopped.");
  newtFinished();
  exit(1);
}


int make_iso_fs(char*scratchdir, char*destfile)
{
  int retval=0,res;
  char old_pwd[MAX_STR_LEN], tmp[MAX_STR_LEN], command[MAX_STR_LEN], message_to_screen[MAX_STR_LEN];
  sprintf(tmp,"make_iso_fs --- scratchdir=%s --- destfile=%s",scratchdir,destfile);
  log_it(tmp);

  getcwd(old_pwd);
  chdir(scratchdir);

  if (CALL_BEFORE_ISO[0]!='\0')
    {
      sprintf(message_to_screen,"Running pre-ISO call for CD#%d",CURR_ISO_NO);
      res=eval_call(CALL_BEFORE_ISO,destfile,CURR_ISO_NO,MONDO_LOGFILE,message_to_screen);
      if (res) { strcat(message_to_screen,"...failed"); } 
      else { strcat(message_to_screen,"...OK"); }
      log_to_screen(message_to_screen);
      retval+=res;
    }

  if (CALL_MAKE_ISO[0]!='\0')
    {
      sprintf(tmp,"%s/archives/NOT-THE-LAST");
      /*
      if (!does_file_exist(tmp))
	{
	  if (CURR_ISO_NO == 1)
	    {sprintf(message_to_screen,"Making first+last ISO (CD #%d)",CURR_ISO_NO);}
	  else
	    {sprintf(message_to_screen,"Making last ISO (CD #%d)",CURR_ISO_NO); }
	}
      else
      	{
      */
 sprintf(message_to_screen,"Making an ISO (CD #%d)",CURR_ISO_NO); 
 /*
   }
 */
      pause_and_ask_for_cdr(2); // if CURR_ISO_NO >= 2 then pause & ask
      res=eval_call(CALL_MAKE_ISO,scratchdir,CURR_ISO_NO,MONDO_LOGFILE,message_to_screen);
      if (res) { strcat(message_to_screen,"...failed"); } 
      else { strcat(message_to_screen,"...OK"); }
      log_to_screen(message_to_screen);
      retval+=res;
    }
  else
    {
      sprintf(message_to_screen,"Running mkisofs to make CD #%d",CURR_ISO_NO);
      log_it(message_to_screen);
      sprintf(tmp,"Call to mkisofs to make ISO (CD #%d) ",CURR_ISO_NO);
      res=eval_call("mkisofs -b images/mindi-boot.2880.img -c boot.cat -o _ISO_ -J -r -p MondoRescue -P www.microwerks.net/~hugo/ -A Mondo_Rescue_GPL -V _CD#_ .",destfile,CURR_ISO_NO,MONDO_LOGFILE,message_to_screen);
      if (res) { strcat(tmp,"...failed"); } else { strcat(tmp,"...OK");}
      log_to_screen(tmp);
      retval+=res;
    }

  if (CALL_BURN_ISO[0]!='\0')
    {
      sprintf(message_to_screen,"Burning ISO (CD #%d)",CURR_ISO_NO);
      pause_and_ask_for_cdr(2);
      res=eval_call(CALL_BURN_ISO,destfile,CURR_ISO_NO,MONDO_LOGFILE,message_to_screen);
      if (res) { strcat(message_to_screen,"...failed"); }
      else { strcat(message_to_screen,"...OK"); }
      log_to_screen(message_to_screen);
      retval+=res;
    }

  if (CALL_AFTER_ISO[0]!='\0')
    {
      sprintf(message_to_screen,"Running post-ISO call (CD #%d)",CURR_ISO_NO);
      res=eval_call(CALL_AFTER_ISO,destfile,CURR_ISO_NO,MONDO_LOGFILE,message_to_screen);
      if (res) { strcat(message_to_screen,"...failed"); }
      else { strcat(message_to_screen,"...OK"); }
      log_to_screen(message_to_screen);
      retval+=res;
    }

  chdir(old_pwd);
  if (retval) { log_it("WARNING - make_iso_fs returned an error"); }
  return(retval);
}


int get_last_filelist_number()
{
  FILE*fin;
  char fname[MAX_STR_LEN],incoming[MAX_STR_LEN+1];
  int res=-1,i;
  sprintf(fname, "%s/LAST-FILELIST-NUMBER",TMP);
  fin=fopen(fname,"r");
  if (fin)
    {
      fgets(incoming,MAX_STR_LEN,fin);
      fclose(fin);
      for(i=strlen(incoming); incoming[i-1]<32; i--);
      incoming[i]='\0';
      res=atoi(incoming);
    }
  return(res);
}



int make_tarballs_and_images(char*scratchdir,char*isodir,long cdsize)
{
  int avtime=120,retval=0, curr_set_no=0,noof_files,res=0, j, percentage;
  long total_compressed_size=0;
  char curr_filelist_fname[MAX_STR_LEN],
    curr_afioball_fname[MAX_STR_LEN], tmp[MAX_STR_LEN],
    files_to_send[ARBITRARY_MAXIMUM][MAX_STR_LEN];

  sprintf(tmp,"%s/archives/filelist.full",scratchdir);
  log_it("make_tarballs_and_images --- starting\n");
  open_progress_form("Backing up filesystem","I am backing up your live filesystem now.","Please wait. This may take a couple of hours.","",get_last_filelist_number()+1);
  sprintf(curr_filelist_fname,"%s/filelist.%d",TMP,0);
  for(curr_set_no=0; does_file_exist(curr_filelist_fname); sprintf(curr_filelist_fname,"%s/filelist.%d",TMP,++curr_set_no))
    {
      // backup this set of files
      sprintf(curr_filelist_fname,"%s/filelist.%d",TMP,curr_set_no);
      sprintf(curr_afioball_fname,"%s/%d.afio.%s",TMP,curr_set_no,ZIPPER_SUFFIX);
      sprintf(tmp,"\nArchiving set %d",curr_set_no);
      log_it(tmp);
      res=tar_this_set(curr_filelist_fname,curr_afioball_fname,curr_set_no);
      retval+=res;
      if (res)
	{
	  sprintf(tmp,"Failed to archive set %d\n",curr_set_no);
	  log_to_screen(tmp); 
	}
      // update screen
      g_current_progress++;
      // sprintf(tmp,"%ld of %ld files have been backed up",g_current_progress,g_maximum_progress);
      percentage=(int)(space_occupied_by_cd(scratchdir)*100/cdsize);
      sprintf(tmp,"CD #%d: [",CURR_ISO_NO);
      for(j=0;j<percentage;j+=5) { strcat(tmp,"*"); }
      for(;j<100;j+=5) { strcat(tmp,"."); }
      sprintf(tmp+strlen(tmp),"] %d%%",percentage);
      update_progress_form(tmp);

      // copy to CD (scratchdir) ... and an actual CD-R if necessary
      total_compressed_size += length_of_file(curr_afioball_fname);
      noof_files=0;
      sprintf(files_to_send[noof_files++],"%s/filelist.%d",TMP,curr_set_no);
      sprintf(files_to_send[noof_files++],"%s/%d.afio.%s",TMP,curr_set_no,ZIPPER_SUFFIX);
      res=add_files_to_cd(scratchdir,isodir,cdsize,files_to_send,noof_files);
      retval+=res;
      if (res) 
	{
	  sprintf(tmp,"Failed to add archive %d's files to CD dir\n",curr_set_no);
	  log_to_screen(tmp);
	}
    }
  close_progress_form();
  sprintf(tmp,"Your regular files have been archived ");
  if (retval) { strcat(tmp,"(with errors)."); }
  else { strcat(tmp,"successfully."); }
  log_to_screen(tmp);
  return(retval);
}




int make_slices_and_images(char*scratchdir, char*isodir, long cdsize, char*biggielist_fname)
{
  FILE*fin;
  char tmp[MAX_STR_LEN], bigfile_fname[MAX_STR_LEN];
  long biggie_file_number=0, noof_biggie_files;
  int retval=0,res=0;
  log_it("make_slices_and_images --- starting");
  sprintf(tmp,"I am now backing up all files >%ldMB in size.",OPTIMAL_SET_SIZE/1024);
  log_to_screen(tmp);
  open_progress_form("Backing up big files",tmp,"Please wait. This may take some time.","",count_lines_in_file(biggielist_fname));
  noof_biggie_files=count_lines_in_file(biggielist_fname);
  fin=fopen(biggielist_fname,"r");
  for(fgets(bigfile_fname,MAX_STR_LEN,fin); !feof(fin); fgets(bigfile_fname,MAX_STR_LEN,fin),biggie_file_number++)
    {
      if (bigfile_fname[strlen(bigfile_fname)-1]<32) {bigfile_fname[strlen(bigfile_fname)-1]='\0';}
      sprintf(tmp,"Bigfile #%ld is '%s'",biggie_file_number,bigfile_fname);
      log_it(tmp);
      if (!does_file_exist(bigfile_fname))
	{ sprintf(tmp,"Warning - %s has disappeared and therefore cannot be sliced up",bigfile_fname); log_it(tmp); }
      else
	{ 
	  res=slice_up_file(bigfile_fname,biggie_file_number,scratchdir,noof_biggie_files); 
	  retval+=res;
	  if (res) { log_to_screen("Warning - failed to slice up fname"); }
	  res=send_slices_to_cd(scratchdir,isodir,cdsize,biggie_file_number);
	  retval+=res;
	  if (res) { log_to_screen("Warning - unable to send slices to CD"); }
	}
      g_current_progress++;
    }
  close_progress_form();
}



int make_those_mondo_archives_dude(char*scratchdir,char*isodir, long cdsize)
{
  char tmp[MAX_STR_LEN],biggielist[MAX_STR_LEN];
  int res=0,retval=0,i;

  /*
  verify_cd_image(isodir);
  return(0);
  */ 

  //  log_to_screen("Backing up your files...");
  sprintf(tmp,"rm -f %s/*.iso",isodir);
  system(tmp);
  wipe_archives(scratchdir);
  log_it("make_those_mondo_archives_dude --- starting");

  // make tarballs
  res=make_tarballs_and_images(scratchdir,isodir,cdsize);
  retval+=res;
  if (res) { log_it("make_tarballs_and_images returned an error"); }

  // slice big files
  sprintf(biggielist,"%s/archives/biggielist.txt",scratchdir);
  sprintf(tmp,"cp %s/biggielist.txt %s",TMP,biggielist);
  log_it(tmp);
  if (system(tmp)) { log_it("...failed"); }
  if (biggielist[strlen(biggielist)-1]<32) {biggielist[strlen(biggielist-1)]='\0';}
  sprintf(tmp,"biggielist = %s",biggielist);
  log_it(tmp);
  res=make_slices_and_images(scratchdir,isodir,cdsize,biggielist);
  retval+=res;
  if (res) { log_it("make_slices_and_images returned an error"); }

  // write final ISO
  res=write_final_iso_if_necessary(scratchdir,isodir,cdsize);
  retval+=res;
  if (res) { log_it("write_final_iso_if_necessary returned an error"); }
  log_it("Dude, those archives have been made. Great.");
  return(retval);
}





void pause_and_ask_for_cdr(int ask_for_one_if_more_than_this)
{
  char tmp[MAX_STR_LEN];
  sprintf(tmp,"I am about to burn CD #%d",CURR_ISO_NO);
  log_it(tmp);
  if (CURR_ISO_NO >= ask_for_one_if_more_than_this)
    {
      sprintf(tmp,"Please insert blank CD #%d",CURR_ISO_NO);
      popup_and_OK(tmp);
    }
  log_it("OK, I assume I have a blank CD in the drive...");
}




int send_slices_to_cd(char*scratchdir,char*isodir,long cdsize,long biggie_file_number)
{
  char biggie_fname[MAX_STR_LEN], tmp[MAX_STR_LEN], list_of_files[ARBITRARY_MAXIMUM][MAX_STR_LEN];
  long the_last_slice, sliceno;
  int retval=0,res;
  strcpy(list_of_files[0],slice_fname(biggie_file_number,0,FALSE,TMP));
  res=add_files_to_cd(scratchdir, isodir, cdsize, list_of_files, 1);
  retval+=res;
  if (res) { log_it("Failed to add 0th slice"); }
  for(the_last_slice=1; !does_file_exist(slice_fname(biggie_file_number,the_last_slice,FALSE,TMP)); the_last_slice++);
  sprintf(tmp,"the_last_slice = %ld",the_last_slice);
  log_it(tmp);
  for(sliceno=1; sliceno<the_last_slice; sliceno++)
    {
      strcpy(list_of_files[0],slice_fname(biggie_file_number,sliceno,TRUE,TMP));
      res=add_files_to_cd(scratchdir, isodir, cdsize, list_of_files, 1);
      unlink(list_of_files[0]);
      retval+=res;
      if (res) 
	{ 
	  sprintf(tmp,"Failed to add slice #%ld of bigfile #%ld",sliceno,biggie_file_number);
	  log_to_screen(tmp);
	  unlink(list_of_files[0]);
	}
    }
  strcpy(list_of_files[0],slice_fname(biggie_file_number,the_last_slice,FALSE,TMP));
  res=add_files_to_cd(scratchdir, isodir, cdsize, list_of_files, 1);
  retval+=res;
  unlink(list_of_files[0]);
  if (res) { log_it("Failed to add final slice"); }
  return(retval);
}





int slice_up_file(char*filename, long biggie_file_number, char*scratchdir, long noof_biggie_files)
{
  char tmp[MAX_STR_LEN], checksum_line[MAX_STR_LEN], command[MAX_STR_LEN], *pB, *pC, tempblock[256*1024];
  FILE*fin,*fout;
  bool finished=FALSE;
  long blksize=0, slice_num=0, i, percentage;
  int retval=0,res=0, j;
  long length;

  if (OPTIMAL_SET_SIZE<999){fatal_error("OPTIMAL_SET_SIZE is insanely small");}
  //  sprintf(tmp,"filename=%s; fileno=%ld",filename,biggie_file_number);
  //  log_it(tmp);
  sprintf(command,"md5sum \"%s\"",filename);
  fin=popen(command,"r");
  fgets(tmp,MAX_STR_LEN,fin);
  pclose(fin);
  pB=strchr(tmp,' ');
  if (pB) { *pB='\0'; }
  sprintf(checksum_line,"%s\t%s",tmp,filename);
  log_it(checksum_line);
  fout=fopen(slice_fname(biggie_file_number,0,FALSE,TMP),"w");
  fprintf(fout,"%s\n",checksum_line);
  fclose(fout);
  length=length_of_file(filename)/OPTIMAL_SET_SIZE/1024;
  fin=fopen(filename,"r");
  for(slice_num=1; !finished; slice_num++)
    {
      fout=fopen(slice_fname(biggie_file_number,slice_num,FALSE,TMP),"w");
      //      sprintf(tmp,"Current file: [",biggie_file_number);
      sprintf(tmp,"File #%ld of %ld: [",biggie_file_number+1,noof_biggie_files);
      if (slice_num==1) {percentage=0;} 
      else {percentage=(int)((slice_num-1)*20/length);}
      for(j=0; j<20 && j<percentage; j++) { strcat(tmp,"*"); }
      for(; j<20; j++) { strcat(tmp,"."); }
      strcat(tmp,"]");
      update_progress_form(tmp);
      for(i=0; i< OPTIMAL_SET_SIZE/256; i++)
	{
	  blksize=fread(tempblock,1,256*1024,fin);
	  if (blksize>0) { fwrite(tempblock,1,blksize,fout); }
	  else { break; }
	}
      fclose(fout);
      if(length_of_file(slice_fname(biggie_file_number,slice_num,FALSE,TMP))>0)
	{
	  sprintf(command,"bzip2 -%d %s",COMPRESSION_LEVEL,slice_fname(biggie_file_number,slice_num,FALSE,TMP));
	  res=system(command);
	  retval+=res;
	  log_it(command);
	  if (res) { log_it("Failed to compress the slice"); }
	}
      else
	{
	  finished=TRUE;
	}
    }
  fclose(fin);
  sprintf(tmp,"Sliced bigfile #%ld",biggie_file_number);
  if (retval) { strcat(tmp,"...FAILED"); }
  else { strcat(tmp,"...OK!"); }
  log_it(tmp);
  return(retval);
}




long space_occupied_by_cd(char *mountpt)
{
  char tmp[MAX_STR_LEN],command[MAX_STR_LEN],*p;
  FILE*fin;
  sprintf(command,"du -sk %s",mountpt);
  fin=popen(command,"r");
  fgets(tmp,MAX_STR_LEN,fin);
  pclose(fin);
  p=strchr(tmp,'\t');
  if (p) {*p='\0';}
  return(atol(tmp));
}




int tar_this_set(char*filelist,char*fname,int setno)
  {
    int retval=0,res=0;
    char command[MAX_STR_LEN],zipparams[MAX_STR_LEN],tmp[MAX_STR_LEN];
    sprintf(zipparams,"-Z -P %s -M 12m -G %d -U -T 3k",ZIPPER_EXECUTABLE,COMPRESSION_LEVEL);
    if (!does_file_exist(filelist))
      {
	sprintf(tmp,"filelist %s does not exist",filelist);
	log_to_screen(tmp);
	return(1);
      }
    sprintf(command,"rm -f %s %s.gz %s.%s",fname,fname,fname,ZIPPER_SUFFIX);
    system(command);
    sprintf(command,"cat %s | afio -o %s %s",filelist,zipparams,fname);

    // log_it("Not afio'ing really... Just dd'ing...");sprintf(command,"dd if=/dev/zero of=%s bs=1k count=%ld",fname,OPTIMAL_SET_SIZE);

    res=run_program_and_log_output(command);
    retval+=res;
    if (res) { 
      log_it(command);
      log_it("...Failed."); 
    }
    return(retval);
}



int write_final_iso_if_necessary(char*scratchdir,char*isodir,long cdsize)
{
  int res;
  char tmp[MAX_STR_LEN];
  /* I should really check if there are any slices or tarballs to be copied to CD-R(W)'s; the odds are approx. 1 in a million that there are no files here, so I'll just go ahead & make one more CD anyway */

  strcpy(tmp,"Writing the final ISO");
  log_it(tmp);
  center_string(tmp,80);
  newtPushHelpLine(tmp);
  res=write_iso_and_go_on(scratchdir,isodir,cdsize,TRUE);
  newtPopHelpLine();
  sprintf(tmp,"Returning from writing final ISO (res=%d)",res);
  log_it(res);
  return(res);
}





int write_iso_and_go_on(char*scratchdir,char*isodir,long cdsize, bool last_cd)
{
  FILE*fout;
  char tmp[MAX_STR_LEN],command[MAX_STR_LEN],cdno_fname[MAX_STR_LEN],lastcd_fname[MAX_STR_LEN], isofile[MAX_STR_LEN];
  bool that_one_was_ok;
  int res=0;

  // label the ISO with its number
  sprintf(cdno_fname,"%s/archives/THIS-CD-NUMBER",scratchdir);
  fout=fopen(cdno_fname,"w");
  fprintf(fout,"%d",CURR_ISO_NO);
  fclose(fout);
  
  // last CD or not? Label accordingly
  sprintf(lastcd_fname,"%s/archives/NOT-THE-LAST",scratchdir);
  if (last_cd)
    {
      unlink(lastcd_fname);
      log_it("OK, you're telling me this is the last CD. Fair enough.");
    }
  else
    {
      fout=fopen(lastcd_fname,"w");
      fprintf(fout,"You're listening to 90.3 WPLN, Nashville Public Radio.\n");
      fclose(fout);
    }
  if (space_occupied_by_cd(scratchdir) > cdsize)
    {
      sprintf(tmp,"Warning! CD is too big. It occupies %ld KB, which is more than the %ld KB allowed.",space_occupied_by_cd(scratchdir),cdsize);
      log_to_screen(tmp);
    }
  sprintf(isofile,"%s/%d.iso",isodir,CURR_ISO_NO);
  for(that_one_was_ok=FALSE ; !that_one_was_ok; )
    {
      res=make_iso_fs(scratchdir,isofile);
      if (VERIFY_LEVEL>0) {res+=verify_cd_image(isodir);}
      if (!res)
	{
	  that_one_was_ok=TRUE;
	}
      else
	{
	  sprintf(tmp,"Failed to burn CD #%d. Retry?",CURR_ISO_NO);
	  res=ask_me_yes_or_no(tmp);
	  if (!res)
	    {
	      if (ask_me_yes_or_no("Abort the backup?"))
		{ fatal_error("FAILED TO BACKUP"); }
	      else
		{ break; }
	    }
	  else
	    { log_it("Retrying, at user's request..."); res=0; }
	}
    }
  CURR_ISO_NO++;
  wipe_archives(scratchdir);
  sprintf(tmp,"rm -Rf %s/images/*gz %s/images/*data*img %s/usr/share/mondo",scratchdir,scratchdir,scratchdir);
  if (system(tmp)) 
    { log_it("Error occurred when I tried to delete the redundant IMGs and GZs"); }
  
  if (last_cd)
    { log_it("This was your last CD."); }
  else
    { log_it("Continuing to backup your data..."); }
  
  return(0);
}

  


void wipe_archives(char*d)
{
  long i;
  char tmp[MAX_STR_LEN], dir[MAX_STR_LEN];
  sprintf(dir,"%s/archives",d);
  sprintf(tmp,"rm -f %s/*.afio*",dir);              system(tmp);
  sprintf(tmp,"rm -f %s/filelist.[0-9]*",dir);      system(tmp);
  sprintf(tmp,"rm -f %s/slice*",dir);               system(tmp);
  sprintf(tmp,"rm -f %s/cklist*",dir);              system(tmp);
  sprintf(tmp,"rm -f %s/zero",dir);                 system(tmp);
  sprintf(tmp,"Wiped %s",dir);
  log_it(tmp);
  sprintf(tmp,"ls -l %s",dir);
  run_program_and_log_output(tmp);
}


char *trim_empty_quotes(char*incoming)
{
  static char outgoing[MAX_STR_LEN];
  char *p;
  if (incoming[0]=='\"' && incoming[strlen(incoming)-1]=='\"')
    {
      strcpy(outgoing,incoming+1);
      outgoing[strlen(outgoing)-1]='\0';
    }
  else
    {
      strcpy(outgoing,incoming);
    }
  return(outgoing);
}



int mount_CDROM_here(char*device,char*mountpoint)
{
  char command[MAX_STR_LEN];
  sprintf(command,"mount %s -o ro -t iso9660 %s 2> /dev/null",device,mountpoint);
  log_it(command);
  return(system(command));
}



int find_cdrom_device(char*output)
{
  FILE*fin;
  bool found_it=FALSE;
  char tmp[MAX_STR_LEN], *p, *q, *r, phrase_one[MAX_STR_LEN],phrase_two[MAX_STR_LEN], command[MAX_STR_LEN];
  output[0]=phrase_one[0]=phrase_two[0]='\0';
  fin=popen("cdrecord -scanbus","r");
  if (!fin) { return(1); }
  for(fgets(tmp,MAX_STR_LEN,fin); !feof(fin); fgets(tmp,MAX_STR_LEN,fin))
    {
      p=strchr(tmp,'\'');
      if (p)
	{
	  q=strchr(++p,'\'');
	  if (q)
	    {
	      for(r=q;*(r-1)==' ';r--);
	      *r='\0';
	      strcpy(phrase_one,p);
	      p=strchr(++q,'\'');
	      if (p)
		{
		  q=strchr(++p,'\'');
		  if (q)
		    {
		      while(*(q-1)==' ') {q--;}
		      *q='\0';
		      strcpy(phrase_two,p);
		    }
		}
	    }
	}
    }
  pclose(fin);
  sprintf(command,"dmesg | grep %s",phrase_two);
  fin=popen(command,"r");
  if (fin)
    {
      for(fgets(tmp,MAX_STR_LEN,fin); !feof(fin); fgets(tmp,MAX_STR_LEN,fin))
	{
	  if (tmp[0]!=' ' && tmp[1]!=' ')
	    {
	      p=strchr(tmp,':');
	      if (p) 
		{
		  *p='\0';
		  sprintf(output,"/dev/%s",tmp);
		  found_it=TRUE;
		}
	    }
	}
      pclose(fin);
    }
  if (found_it) { return(0); } else { return(1); }
}



char *strip_afio_output_line(char*input)
{
  static char output[MAX_STR_LEN];
  char *p, *q;
  strcpy(output,input);
  p=strchr(input,'\"');
  if (p)
    {
      q=strchr(++p,'\"');
      if (q)
	{
	  strcpy(output,p);
	  *(strchr(output,'\"'))='\0';
	}
    }
  return(output);
}



char *vfy_tball_fname(char*mountpoint,int setno)
{
  static char output[MAX_STR_LEN];
  sprintf(output,"%s/archives/%d.afio.%s",mountpoint,setno,ZIPPER_SUFFIX);
  return(output);
}



int verify_a_tarball(char*mountpoint,int set_number)
{
  char old_pwd[MAX_STR_LEN], command[MAX_STR_LEN], outfile[MAX_STR_LEN], tmp[MAX_STR_LEN], *p, *q;
  FILE*fin;
  long diffs=0;
  getcwd(old_pwd);
  chdir("/");
  sprintf(outfile,"%s/afio.log",TMP);
  sprintf(command,"afio -r -P %s -Z %s &> %s", ZIPPER_EXECUTABLE, vfy_tball_fname(mountpoint,set_number),outfile);
  system(command);
  sprintf(command,"cat %s | cut -d':' -f2 | sort | uniq",outfile);
  fin=popen(command,"r");
  if (fin)
    {
      for(fgets(tmp,MAX_STR_LEN,fin); !feof(fin); fgets(tmp,MAX_STR_LEN,fin))
	{
	  if (!diffs)
	    {
	      sprintf(tmp,"Files in archive #%d that differ from filesystem: \n",set_number);
	      log_it(tmp);
	    }
	  diffs++;
	  sprintf(tmp,"%s\n",strip_afio_output_line(tmp));
	  log_it(tmp);
	}
      pclose(fin);
      sprintf(tmp,"ISO=%d  archive=%d  diffs=%ld",CURR_ISO_NO,set_number,diffs);
      if (diffs) { log_to_screen(tmp); }
      else { log_it(tmp); }
    }
  chdir(old_pwd);
  return(0);
}



int g_last_tarball_number=-1;

int verify_all_tarballs(char*mountpoint)
{
  char tmp[MAX_STR_LEN], current_tarball_fname[MAX_STR_LEN];
  int set_number, retval, res, total_sets, percentage;
  for(set_number=0; set_number < 9999 && !does_file_exist(vfy_tball_fname(mountpoint,set_number)); set_number++);
  if (!does_file_exist(vfy_tball_fname(mountpoint,set_number))) 
    {
      return(0);
    }
  if (g_last_tarball_number!=set_number-1)
    {
      retval++;
      sprintf(tmp,"Warning - missing set(s) between %d and %d\n",g_last_tarball_number,set_number-1);
      log_to_screen(tmp);
    }
  sprintf(tmp,"Verifying ISO #%d's tarballs",CURR_ISO_NO);
  open_evalcall_form(tmp);
  for(total_sets=set_number; does_file_exist(vfy_tball_fname(mountpoint,total_sets)); total_sets++);
  for(; does_file_exist(vfy_tball_fname(mountpoint,set_number));set_number++)
    {
      percentage = (set_number - g_last_tarball_number) * 100 / (total_sets - g_last_tarball_number);
      update_evalcall_form(percentage);
      retval+=verify_a_tarball(mountpoint,set_number);
    }
  g_last_tarball_number=set_number-1;
  close_evalcall_form();
  return(retval);
}



long last_bigfile_num=-1, last_slice_num=-1;

int verify_all_slices(char*mtpt)
{
  char tmp[MAX_STR_LEN], mountpoint[MAX_STR_LEN];
  long bigfile_num, slice_num;
  sprintf(tmp,"Verifying ISO#%d's big files",CURR_ISO_NO);
  open_evalcall_form(tmp);
  sprintf(mountpoint,"%s/archives",mtpt);
  if (last_bigfile_num==-1)
    { 
      bigfile_num=0;
      slice_num=0;
    }
  else if (slice_num==0)
    {
      bigfile_num=last_bigfile_num+1;
      slice_num=0;
    }
  else
    {
      bigfile_num=last_bigfile_num; 
      slice_num=last_slice_num+1;
    }
//printf("OK, bigfile_num = %ld and slice_num = %ld\n",bigfile_num,slice_num);
  while(does_file_exist(slice_fname(bigfile_num,slice_num,TRUE,mountpoint)) ||
	does_file_exist(slice_fname(bigfile_num,slice_num,FALSE,mountpoint)))
    {
      if (slice_num==0) 
	{
	  sprintf(tmp,"ISO=%d  bigfile=%ld --START--",CURR_ISO_NO,bigfile_num); 
	  log_it(tmp);
	  slice_num++;
	}
      else if (does_file_exist(slice_fname(bigfile_num,slice_num,FALSE,mountpoint)))
	{
	  sprintf(tmp,"ISO=%d  bigfile=%ld ---END---",CURR_ISO_NO,bigfile_num);
	  log_it(tmp);
	  bigfile_num++;
	  slice_num=0;
	}
      else
	{
	  sprintf(tmp,"ISO=%d  bigfile=%ld  slice=%ld  \r",CURR_ISO_NO,bigfile_num,slice_num);	  
	  log_it(tmp);
	  slice_num++;
	}
    }
  last_bigfile_num=bigfile_num;
  last_slice_num=slice_num-1;
  if (last_slice_num < 0) { last_bigfile_num--; }
  close_evalcall_form();
  return(0);
}



char g_cdrom_device[MAX_STR_LEN]="";



int find_and_mount_actual_cd(char*mountpoint)
{
  char tmp[MAX_STR_LEN], command[MAX_STR_LEN];
  int retval=0,res;
  if (g_cdrom_device[0]=='\0')
    {
      res=find_cdrom_device(g_cdrom_device);
      sprintf(tmp,"g_cdrom_device = %s",g_cdrom_device);
      log_it(tmp);
      if (res)
	{
	  if (!popup_and_get_string("CD-RW device","Please enter your CD-RW's /dev device",g_cdrom_device))
	    { res=1; }
	  else
	    { res=0; }
	}
    }
  if (res) { return(res); }
  if (mount_CDROM_here(g_cdrom_device,mountpoint))
    {
      strcpy(g_cdrom_device,"/dev/scd0");
      if (res=mount_CDROM_here(g_cdrom_device,mountpoint))
	{
	  strcpy(g_cdrom_device,"/dev/sr0");
	  if (res=mount_CDROM_here(g_cdrom_device,mountpoint))
	    {
	      strcpy(g_cdrom_device,"/dev/scd1");
	      if (res=mount_CDROM_here(g_cdrom_device,mountpoint))
		{
		  strcpy(g_cdrom_device,"/dev/sr1");
		  if (res=mount_CDROM_here(g_cdrom_device,mountpoint))
		    {
		      if (!popup_and_get_string("CDRW device","Please enter your CDRW's /dev device"),g_cdrom_device)
			{ res=1; }
		      else
			{ res=0; }
		    }
		}
	    }
	}
    }
  return(res);
}



int verify_cd_image(char*isodir)      // also uses TMP and CURR_ISO_NO
{
  int retval=0,res;
  char mountpoint[MAX_STR_LEN], command[MAX_STR_LEN], tmp[MAX_STR_LEN], fname[MAX_STR_LEN];
  sprintf(mountpoint,"%s/cdrom",TMP);
  sprintf(fname,"%s/%d.iso",isodir,CURR_ISO_NO);
  mkdir(mountpoint);
  if (!does_file_exist(fname)) 
    {
      if (find_and_mount_actual_cd(mountpoint))
	{
	  log_to_screen("failed to mount actual CD");
	  return(1);
	}
    }
  else
    {
      sprintf(command,"mount -o loop,ro -t iso9660 %s %s",fname,mountpoint);
      if (system(command))
	{
	  sprintf(tmp,"%s failed; unable to mount ISO image\n",command);
	  log_to_screen(tmp);
	  return(1);
	}
    }
  log_it("OK, I've mounted the ISO/CD\n");
  //  retval+=
  verify_all_tarballs(mountpoint);
  //  retval+=
  verify_all_slices(mountpoint);
  sprintf(command,"umount %s",mountpoint);
  if (system(command))
    {
      sprintf(tmp,"%s failed; unable to unmount ISO image\n",command);
      log_to_screen(tmp);
      retval++;
    }
  else
    {
     log_it("OK, I've unmounted the ISO file\n");
    }
  if (!does_file_exist(fname))
    {
      sprintf(tmp,"eject %s",g_cdrom_device);
      if (system(tmp)) { log_it("Failed to eject CD-ROM drive"); }
    }
  return(retval);
}



int make_tape_archives(char *scratchdir)
{
  char tmp[MAX_STR_LEN], command[MAX_STR_LEN], filelist_fname[MAX_STR_LEN],
    incoming[MAX_STR_LEN], stdout_fname[MAX_STR_LEN], 
    lockfile[MAX_STR_LEN], pause_for_tape_script[MAX_STR_LEN],
    pause_for_tape_lkfile[MAX_STR_LEN];
  int i, curr_tape_no=1;
  long total_noof_files;
  sprintf(filelist_fname,"%s/archives/filelist.full",scratchdir);
  if (!does_file_exist(filelist_fname)) {fatal_error("Cannot find filelist");}
  total_noof_files=count_lines_in_file(filelist_fname);
  open_progress_form("Backing up filesystem","I am backing up your live filesystem to tape.", "Please wait. This may take a couple of hours.","",total_noof_files);
  sprintf(lockfile, "%s/tony-blair-is-mojo-jojo",TMP);
  sprintf(stdout_fname, "%s/stdout.txt",TMP);
  sprintf(pause_for_tape_script, "%s/pause-for-next-tape",MONDO_PATH);
  if (!does_file_exist(pause_for_tape_script))
    { fatal_error("pause_for_tape_script pause script does not exist"); }
  sprintf(pause_for_tape_lkfile, "/tmp/mojo-jojo-pauses-for-next-tape");
  sprintf(command,"sh -c \"echo hi > %s; cat %s | afio -v -o -b 10k -f -s %ldm -M 12m -c 4096 -Z -G %d -T 4k -P %s -H %s %s &> %s; rm -f %s\" &", lockfile, filelist_fname, TAPE_SIZE, COMPRESSION_LEVEL, ZIPPER_EXECUTABLE, pause_for_tape_script, TAPE_DEVICE, stdout_fname, lockfile);
  sprintf(tmp,"calling '%s' to archive to tape",command);
  log_it(tmp);
  if (system(command)) { log_to_screen("Failed to execute afio; failed to backup to tape"); return(1); }
  sleep(1);
  update_progress_form("Compressing, buffering, archiving...");
  for(i=0; i<30 && !does_file_exist(lockfile); i++) { sleep(1); }
  if (!does_file_exist(lockfile))
    {
      log_to_screen("Something went seriously wrong with the call to afio.");
      sprintf(tmp,"Last error: '%s'",last_line_of_file(stdout_fname));
      log_it(tmp);
      log_it("Output of call to afio:-");
      sprintf(tmp,"cat %s >> %s",stdout_fname,MONDO_LOGFILE);
      system(tmp);
      log_it("(End of output)");
      return(1);
    }
  sprintf(tmp,"Backing up to tape #%d...",curr_tape_no);
  log_to_screen(tmp);
  while (does_file_exist(lockfile))
    {
      log_file_end_to_screen(stdout_fname,"");
      sleep(2);
      g_current_progress = count_lines_in_file(stdout_fname);
      if (g_current_progress > g_maximum_progress)
	{
	  g_current_progress = g_maximum_progress; 
	}
      sprintf(tmp,"Processed %ld files so far. Writing tape #%d...",g_current_progress,curr_tape_no);
      update_progress_form(tmp);
      if (does_file_exist(pause_for_tape_lkfile))
	{
	  system("sync");
	  sleep(2);
	  curr_tape_no++;
	  sprintf(tmp,"When the current tape has stopped whirring, please insert tape #%d.",curr_tape_no);
	  popup_and_OK(tmp);
	  unlink(pause_for_tape_lkfile);
	  sprintf(tmp,"echo 'Backing up to tape #%d...' >> %s",curr_tape_no,stdout_fname);
	  system(tmp);
	}
    }
  if (VERIFY_LEVEL > 0)
    { popup_and_OK("Mondo does not yet know how to verify tape archives."); }
  close_progress_form();
  return(0);
}



/* -----------------------------main-------------------------- */



int main(int argc, char*argv[])
{

  char scratchdir[MAX_STR_LEN],
    archivepath[MAX_STR_LEN],
    isodir[MAX_STR_LEN],
    tmp[MAX_STR_LEN];
  long cdsize;
  int argno=1,res,j,i;

  /*
  system("mkdir -p /tmp/tmp.mondo");
  strcpy(TMP,"/tmp/tmp.mondo");
  res=0;
  exit(res);
  */

  if (argc!=21)
    {
      printf("Wrong number of params passed; should be 21, was %d\n",argc);
      //      exit(1);
      
      sprintf(tmp,"Entering DEBUG MODE");    printf("%s\n",tmp);
      log_it(tmp);                           strcpy(archivepath,"/blah");
      strcpy(scratchdir,"/home/mondo.scratch"); strcpy(isodir,"/home");
      cdsize=100*1024;                       strcpy(TMP,"/home/tmp.mondo.999");
      strcpy(MONDO_LOGFILE,"/tmp/mondo-archive.log");
      COMPRESSION_LEVEL=1;                   VERIFY_LEVEL=0;
      DEBUGGING=TRUE;                        OPTIMAL_SET_SIZE=8192;
      RUNNING_LIVE=TRUE;                     CALL_BEFORE_ISO[0]='\0';
      CALL_MAKE_ISO[0]='\0';                 CALL_BURN_ISO[0]='\0';
      CALL_AFTER_ISO[0]='\0';                strcpy(STARTDIR,"/home");
      strcpy(MONDO_PATH,"/usr/share/mondo"); strcpy(TAPE_DEVICE,"/dev/st0");
      TAPE_SIZE=10; // megabytes
      sprintf(tmp,"find /usr/share/doc > %s/filelist.full",TMP);
      if (system(tmp)) { log_it("failed to generate simple filelist");exit(1);}
    }
  else
    {
      strcpy(archivepath,trim_empty_quotes(argv[argno++]));
      strcpy(scratchdir,trim_empty_quotes(argv[argno++]));
      strcpy(isodir,trim_empty_quotes(argv[argno++]));
      cdsize=atol(trim_empty_quotes(argv[argno++]));
      strcpy(TMP,trim_empty_quotes(argv[argno++]));
      strcpy(MONDO_LOGFILE,trim_empty_quotes(argv[argno++]));
      COMPRESSION_LEVEL = atoi(trim_empty_quotes(argv[argno++]));
      if (!strcmp(argv[argno++],"yes")) {USE_LZO=TRUE;} else {USE_LZO=FALSE;}
      VERIFY_LEVEL = atoi(trim_empty_quotes(argv[argno++]));
      if (!strcmp(argv[argno++],"yes")) {DEBUGGING=TRUE;} else {DEBUGGING=FALSE;}
      OPTIMAL_SET_SIZE = atol(trim_empty_quotes(argv[argno++]));
      if (!strcmp(argv[argno++],"yes")) {RUNNING_LIVE=TRUE;} else {RUNNING_LIVE=FALSE;}
      strcpy(CALL_BEFORE_ISO,trim_empty_quotes(argv[argno++]));
      strcpy(CALL_MAKE_ISO,trim_empty_quotes(argv[argno++]));
      strcpy(CALL_AFTER_ISO,trim_empty_quotes(argv[argno++]));
      strcpy(CALL_BURN_ISO,trim_empty_quotes(argv[argno++]));
      strcpy(STARTDIR,trim_empty_quotes(argv[argno++]));
      strcpy(MONDO_PATH,trim_empty_quotes(argv[argno++]));
      strcpy(TAPE_DEVICE,trim_empty_quotes(argv[argno++]));
      TAPE_SIZE=atol(trim_empty_quotes(argv[argno++]));
      CURR_ISO_NO=1;
      cdsize=(cdsize-1)*1024;
    }

  if (USE_LZO) 
    { 
      strcpy(ZIPPER_EXECUTABLE,"lzop"); 
      strcpy(ZIPPER_SUFFIX,"lzo");
    }
  else 
    { 
      strcpy(ZIPPER_EXECUTABLE,"bzip2"); 
      strcpy(ZIPPER_SUFFIX,"bz2");
    }
  setup_newt_stuff();
  sprintf(tmp,"Noof args = %d",argc);
  log_it(tmp);
  for(i=1;i<argc;i++) { log_it(argv[i]); }
  log_it("---end of args---");

  /*
  open_progress_form("Hi there","This is the background form","Yes it is","",100);
  sleep(1);
  sprintf(tmp,"Device=%s; size=%ld",TAPE_DEVICE,TAPE_SIZE);
  log_to_screen(tmp);
  open_evalcall_form("Writing tape #1");
  for(i=0;i<100;i+=20)
    { 
      update_evalcall_form(i); 
      sleep(1); 
    }
  close_evalcall_form();
  sleep(1);
  close_progress_form();
  newtFinished();
  exit(1);
  */

  if (TAPE_DEVICE[0]!='\0')
    {
      res=make_tape_archives(scratchdir);
    }
  else if (archivepath[0]=='\0')
    {
      sprintf(tmp,"echo blah > %s/NO-TARBALLS-HERE",scratchdir);
      system(tmp);
      sprintf(tmp,"%s/rescue.iso",isodir);
      res=make_iso_fs(scratchdir,tmp);
    }
  else
    {
      res=make_those_mondo_archives_dude(scratchdir,isodir,cdsize);
    }

  if (res)
    {
      log_it("Warnings/errors occurred.");
    }
  else
    {
      log_it("No warnings or errors were reported. Life is good. :-)");
    }

  newtFinished();
  exit(res);
}
